@@ -4,7 +4,7 @@
  * This file is part of msmtp, an SMTP client.
  *
  * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
- * 2014, 2016
+ * 2014, 2016, 2018, 2019
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
@@ -48,6 +48,7 @@
 #include "xalloc.h"
 #include "list.h"
 #include "readbuf.h"
+#include "tools.h"
 #include "net.h"
 #include "smtp.h"
 #include "stream.h"
@@ -128,12 +129,12 @@ smtp_server_t smtp_new(FILE *debug, int protocol)
  */
 
 int smtp_connect(smtp_server_t *srv, const char *proxy_host, int proxy_port,
-        const char *host, int port, int timeout,
+        const char *host, int port, const char *source_ip, int timeout,
         char **server_canonical_name, char **server_address,
         char **errstr)
 {
-    return net_open_socket(proxy_host, proxy_port, host, port, timeout, &srv->fd,
-            server_canonical_name, server_address, errstr);
+    return net_open_socket(proxy_host, proxy_port, host, port, source_ip,
+            timeout, &srv->fd, server_canonical_name, server_address, errstr);
 }
 
 
@@ -599,10 +600,10 @@ int smtp_tls_starttls(smtp_server_t *srv, list_t **error_msg, char **errstr)
 
 #ifdef HAVE_TLS
 int smtp_tls(smtp_server_t *srv, const char *hostname, int tls_nocertcheck,
-        tls_cert_info_t *tci, char **errstr)
+        tls_cert_info_t *tci, char **tls_parameter_description, char **errstr)
 {
     return tls_start(&srv->tls, srv->fd, hostname, tls_nocertcheck, tci,
-            errstr);
+            tls_parameter_description, errstr);
 }
 #endif /* HAVE_TLS */
 
@@ -1029,24 +1030,24 @@ int smtp_auth(smtp_server_t *srv,
     {
         /* Choose "best" authentication mechanism. */
         /* TODO: use gsasl_client_suggest_mechanism()? */
-        if (gsasl_client_support_p(ctx, "SCRAM-SHA-1")
-                && (srv->cap.flags & SMTP_CAP_AUTH_SCRAM_SHA_1))
-        {
-            auth_mech = "SCRAM-SHA-1";
-        }
-        else if (gsasl_client_support_p(ctx, "CRAM-MD5")
-                && (srv->cap.flags & SMTP_CAP_AUTH_CRAM_MD5))
-        {
-            auth_mech = "CRAM-MD5";
-        }
 #ifdef HAVE_TLS
-        else if (tls_is_active(&srv->tls))
+        if (tls_is_active(&srv->tls))
         {
             if (gsasl_client_support_p(ctx, "PLAIN")
                     && (srv->cap.flags & SMTP_CAP_AUTH_PLAIN))
             {
                 auth_mech = "PLAIN";
             }
+            else if (gsasl_client_support_p(ctx, "SCRAM-SHA-1")
+                    && (srv->cap.flags & SMTP_CAP_AUTH_SCRAM_SHA_1))
+            {
+                auth_mech = "SCRAM-SHA-1";
+            }
+            else if (gsasl_client_support_p(ctx, "CRAM-MD5")
+                    && (srv->cap.flags & SMTP_CAP_AUTH_CRAM_MD5))
+            {
+                auth_mech = "CRAM-MD5";
+            }
             else if (gsasl_client_support_p(ctx, "DIGEST-MD5")
                     && (srv->cap.flags & SMTP_CAP_AUTH_DIGEST_MD5))
             {
@@ -1063,7 +1064,15 @@ int smtp_auth(smtp_server_t *srv,
                 auth_mech = "NTLM";
             }
         }
+        else
 #endif /* HAVE_TLS */
+        {
+            if (gsasl_client_support_p(ctx, "SCRAM-SHA-1")
+                    && (srv->cap.flags & SMTP_CAP_AUTH_SCRAM_SHA_1))
+            {
+                auth_mech = "SCRAM-SHA-1";
+            }
+        }
     }
     if (strcmp(auth_mech, "") == 0)
     {
@@ -1312,17 +1321,17 @@ int smtp_auth(smtp_server_t *srv,
     if (strcmp(auth_mech, "") == 0)
     {
         /* Choose "best" authentication mechanism. */
-        if (srv->cap.flags & SMTP_CAP_AUTH_CRAM_MD5)
-        {
-            auth_mech = "CRAM-MD5";
-        }
 #ifdef HAVE_TLS
-        else if (tls_is_active(&srv->tls))
+        if (tls_is_active(&srv->tls))
         {
             if (srv->cap.flags & SMTP_CAP_AUTH_PLAIN)
             {
                 auth_mech = "PLAIN";
             }
+            else if (srv->cap.flags & SMTP_CAP_AUTH_CRAM_MD5)
+            {
+                auth_mech = "CRAM-MD5";
+            }
             else if (srv->cap.flags & SMTP_CAP_AUTH_LOGIN)
             {
                 auth_mech = "LOGIN";
@@ -1907,3 +1916,31 @@ void smtp_close(smtp_server_t *srv)
 #endif /* HAVE_TLS */
     net_close_socket(srv->fd);
 }
+
+
+/*
+ * smtp_exitcode()
+ *
+ * see smtp.h
+ */
+
+int smtp_exitcode(int smtp_error_code)
+{
+    switch (smtp_error_code)
+    {
+        case SMTP_EIO:
+            return EX_IOERR;
+        case SMTP_EPROTO:
+            return EX_PROTOCOL;
+        case SMTP_EINVAL:
+            return EX_DATAERR;
+        case SMTP_EAUTHFAIL:
+            return EX_NOPERM;
+        case SMTP_EINSECURE:
+        case SMTP_EUNAVAIL:
+            return EX_UNAVAILABLE;
+        case SMTP_ELIBFAILED:
+        default:
+            return EX_SOFTWARE;
+    }
+}
